
# ESA (C) 2000-2018 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
## @file
# Parse XML config file

#========================================================================
use XML::LibXML;
use IO::Handle;

use GVEPICExposure;
use GVTask;
use GVProduct;
use GVODF;



@exposures = ();
$odf_object = new GVODF();

# initialize the parser
 
# open a filehandle and parse
print "Trying to open file: \n";
my $parser = XML::LibXML->new();
my $filename = "MyFile.xml";
my $doc    = $parser->parse_file($filename);

#print "file opened: \n";
#$elname = $rootel -> getName();
#print "Root element is a $elname and it contains ...\n";

my %dist;


&proc_node( $doc->getDocumentElement, \%dist );

print "size... $#exposures\n";

print "Showing Info...\n";
$odf_object->print();
for ($i=0;$i<=$#exposures;$i++)
{
   	$expo = $exposures[$i];
   	$expo->print();
  
   	$products = $expo->getProducts();
   	$id = $expo->getExpId();
   
   	print "Product size: $#{$products}\n";
   
   	for ($j=0;$j<=$#{$products};$j++)
   	{
	   	$gh = @{$products}[$j];
	   	$pName = $gh->getProduct();
   		print "Product Name: $pName\n";

	   	$tasks = $gh->getTasks();
   		print "Task size: $#{$tasks}\n";
   		for ($jj=0;$jj<=$#{$tasks};$jj++)
   		{
	   		$task = @{$tasks}[$jj];
	   		$tName = $task->getTask();
	   		print "Task Name: $tName\n";
   			$p = $task->getParams();
   			while (($key, $value) = each(%$p)){
	    		print "MEPARTO!!!***.. ".$key."=".$value."\n";
			}
  		}	   
  	}
      }

print $doc->toString;

# process an XML tree node: if it's an element, update the
# distribution list and process all its children
#

sub proc_node {
    my( $node, $dist ) = @_;
    return unless( $node->nodeType eq &XML_ELEMENT_NODE );
    $dist->{ $node->nodeName } ++;
    

    foreach my $child ( $node->getChildnodes ) {
   		if($child -> nodeType ne &XML_TEXT_NODE)
    	{
#    		$elname = $child -> getName();
#        	@atts = $child -> getAttributes();
			          
        	foreach my $obs ($child->findnodes('OBSERVATION')) {
        		foreach my $param ($obs->findnodes('PARAM')) {
	            	$id = $param->getAttribute('id');
	            	$value = $param->getAttribute('default');
	            	if ($id eq "analysisoption") {
	            		$odf_object->setAnalysisOption($value);
	            	} elsif ($id eq "epicsrc") {
	            		$odf_object->setEPICsrc($value)
	            	}  elsif ($id eq "ra") {
	            		$odf_object->setRA($value)
	            	}  elsif ($id eq "dec") {
	            		$odf_object->setDEC($value)
	            	}  elsif ($id eq "sourcename") {
	            		$odf_object->setSourceName($value)
	            	}  elsif ($id eq "obsid") {
	            		$odf_object->setObsId($value)
	            	}	elsif ($id eq "EPN") {
	            		$odf_object->setEPN($value)
	            	}	elsif ($id eq "EMOS1") {
	            		$odf_object->setEMOS1($value)
	            	}	elsif ($id eq "EMOS2") {
	            		$odf_object->setEMOS2($value)
	            	}	elsif ($id eq "RGS1") {
	            		$odf_object->setRGS1($value)
	            	}	elsif ($id eq "RGS2") {
	            		$odf_object->setRGS2($value)
	            	}	elsif ($id eq "OM") {
	            		$odf_object->setOM($value)
	            	}	            		                 		            		            	
        		}
        	} 
        	        		
        	foreach my $inst ($child->findnodes('INSTRUMENT')) {
            	my $instName = $inst->getAttribute('value');                	
        	  
   
	        	foreach my $expo ($inst->findnodes('EXPOSURE')) {
	        		my $object = new GVEPICExposure();
	
					$object->setInstName($instName);
		        	$object->setMode($expo->getAttribute('mode'));
		        	$id = $expo->getAttribute('expid');
		  	       	$object->setExpId($expo->getAttribute('expid'));
		           	$object->setDuration($expo->getAttribute('duration'));
		           	$object->setProcess($expo->getAttribute('process'));
					my @products = ();
		 			print "Reading Exposure...: $instName ID: $id\n";
		 			foreach my $product ($expo->findnodes('PRODUCT'))
		 			{
		 				$name = $product->getAttribute('value');
		 				print "Reading product: $name\n";
		 				$productObj = new GVProduct();
		 				$productObj->setProduct($product->getAttribute('value'));
		 				$productObj->setProcess($product->getAttribute('process'));
		 				
		 				my @tasks = ();	
						foreach my $task ($product->findnodes("TASK")) {				
						    $k = $task->getAttribute('name');
			        		print "Reading Tarea $k\n";
			        		my $taskObj = new GVTask();
			        		my $params = {};
			        	
			        		$taskObj->setTask($task->getAttribute('name'));
							$taskObj->setInfo($task->getAttribute('purpose'));
			        		foreach my $param ($task->findnodes('PARAM')) {        		 
			          			if ($param->hasAttribute('default') ) {
				          			$params->{$param->getAttribute('id')} = $param->getAttribute('default');         	 		
			         			}         			         		
			        		}	
			        		$taskObj->setParam($params);
			        	
			        		print "Task done\n";      
	
			        		push(@tasks,$taskObj);  
						} 					       
						$productObj->setTasks(\@tasks);
			           	push(@products,$productObj);
			           	
			        }
			        print "Product done\n";		       		  
			        $object->setProducts(\@products);
			        push(@exposures,$object);				     
	        	}
    		}
    
    	 
        &proc_node( $child, $dist );
    }

    }
}

1;
